@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2000-2002  Maxim Krasnyansky <maxk@qualcomm.com>
- *  Copyright (C) 2003-2006  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2003-2007  Marcel Holtmann <marcel@holtmann.org>
  *
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -50,7 +50,7 @@ static inline uint16_t get_manufacturer(void)
 	return (manufacturer == DEFAULT_COMPID ? parser.defcompid : manufacturer);
 }
 
-#define EVENT_NUM 55
+#define EVENT_NUM 61
 static char *event_str[EVENT_NUM + 1] = {
 	"Unknown",
 	"Inquiry Complete",
@@ -100,6 +100,7 @@ static char *event_str[EVENT_NUM + 1] = {
 	"Synchronous Connect Changed",
 	"Sniff Subrate",
 	"Extended Inquiry Result",
+	"Encryption Key Refresh Complete",
 	"Unknown",
 	"Unknown",
 	"Unknown",
@@ -108,9 +109,14 @@ static char *event_str[EVENT_NUM + 1] = {
 	"Unknown",
 	"Unknown",
 	"Link Supervision Timeout Change",
+	"Enhanced Flush Complete",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Remote Host Supported Features Notification",
 };
 
-#define CMD_LINKCTL_NUM 41
+#define CMD_LINKCTL_NUM 49
 static char *cmd_linkctl_str[CMD_LINKCTL_NUM + 1] = {
 	"Unknown",
 	"Inquiry",
@@ -144,7 +150,7 @@ static char *cmd_linkctl_str[CMD_LINKCTL_NUM + 1] = {
 	"Read Remote Version Information",
 	"Unknown",
 	"Read Clock Offset",
-	"Read LMP Handle"
+	"Read LMP Handle",
 	"Unknown",
 	"Unknown",
 	"Unknown",
@@ -155,6 +161,13 @@ static char *cmd_linkctl_str[CMD_LINKCTL_NUM + 1] = {
 	"Setup Synchronous Connection",
 	"Accept Synchronous Connection",
 	"Reject Synchronous Connection",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
 };
 
 #define CMD_LINKPOL_NUM 17
@@ -176,10 +189,10 @@ static char *cmd_linkpol_str[CMD_LINKPOL_NUM + 1] = {
 	"Read Default Link Policy Settings",
 	"Write Default Link Policy Settings",
 	"Flow Specification",
-	"Sniff Subrate",
+	"Sniff Subrating",
 };
 
-#define CMD_HOSTCTL_NUM 82
+#define CMD_HOSTCTL_NUM 95
 static char *cmd_hostctl_str[CMD_HOSTCTL_NUM + 1] = {
 	"Unknown",
 	"Set Event Mask",
@@ -264,6 +277,20 @@ static char *cmd_hostctl_str[CMD_HOSTCTL_NUM + 1] = {
 	"Unknown",
 	"Read Extended Inquiry Response",
 	"Write Extended Inquiry Response",
+	"Refresh Encryption Key",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Read Inquiry Response Transmit Power Level",
+	"Write Inquiry Response Transmit Power Level",
+	"Read Default Erroneous Data Reporting",
+	"Write Default Erroneous Data Reporting",
+	"Unknown",
+	"Unknown",
+	"Unknown",
+	"Enhanced Flush"
+	"Unknown",
 };
 
 #define CMD_INFO_NUM 9
@@ -292,7 +319,16 @@ static char *cmd_status_str[CMD_STATUS_NUM + 1] = {
 	"Read Clock",
 };
 
-#define ERROR_CODE_NUM 53
+#define CMD_TESTING_NUM 4
+static char *cmd_testing_str[CMD_TESTING_NUM + 1] = {
+	"Unknown",
+	"Read Loopback Mode",
+	"Write Loopback Mode",
+	"Enable Device Under Test mode",
+	"Unknown",
+};
+
+#define ERROR_CODE_NUM 56
 static char *error_code_str[ERROR_CODE_NUM + 1] = {
 	"Success",
 	"Unknown HCI Command",
@@ -348,6 +384,9 @@ static char *error_code_str[ERROR_CODE_NUM + 1] = {
 	"Reserved",
 	"Reserved Slot Violation",
 	"Role Switch Failed",
+	"Extended Inquiry Response Too Large",
+	"Unknown",
+	"Unknown",
 };
 
 static char *status2str(uint8_t status)
@@ -405,7 +444,10 @@ static char *opcode2str(uint16_t opcode)
 		break;
 
 	case OGF_TESTING_CMD:
-		cmd = "Testing";
+		if (ocf <= CMD_TESTING_NUM)
+			cmd = cmd_testing_str[ocf];
+		else
+			cmd = "Unknown";
 		break;
 
 	case OGF_VENDOR_CMD:
@@ -709,6 +751,15 @@ static inline void read_remote_ext_features_dump(int level, struct frame *frm)
 	printf("handle %d page %d\n", btohs(cp->handle), cp->page_num);
 }
 
+static inline void setup_sync_conn_dump(int level, struct frame *frm)
+{
+	setup_sync_conn_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d voice setting 0x%4.4x\n", btohs(cp->handle),
+						btohs(cp->voice_setting));
+}
+
 static inline void hold_mode_dump(int level, struct frame *frm)
 {
 	hold_mode_cp *cp = frm->ptr;
@@ -728,6 +779,25 @@ static inline void sniff_mode_dump(int level, struct frame *frm)
 		btohs(cp->min_interval), btohs(cp->attempt), btohs(cp->timeout));
 }
 
+static inline void qos_setup_dump(int level, struct frame *frm)
+{
+	qos_setup_cp *cp = frm->ptr;
+
+	p_indent(level, frm);
+	printf("handle %d flags 0x%2.2x\n", btohs(cp->handle), cp->flags);
+
+	p_indent(level, frm);
+	printf("Service type: %d\n", cp->qos.service_type);
+	p_indent(level, frm);
+	printf("Token rate: %d\n", btohl(cp->qos.token_rate));
+	p_indent(level, frm);
+	printf("Peak bandwith: %d\n", btohl(cp->qos.peak_bandwidth));
+	p_indent(level, frm);
+	printf("Latency: %d\n", btohl(cp->qos.latency));
+	p_indent(level, frm);
+	printf("Delay variation: %d\n", btohl(cp->qos.delay_variation));
+}
+
 static inline void write_link_policy_dump(int level, struct frame *frm)
 {
 	write_link_policy_cp *cp = frm->ptr;
@@ -964,6 +1034,18 @@ static inline void write_inquiry_mode_dump(int level, struct frame *frm)
 	printf("mode %d\n", cp->mode);
 }
 
+static inline void set_afh_classification_dump(int level, struct frame *frm)
+{
+	set_afh_classification_cp *cp = frm->ptr;
+	int i;
+
+	p_indent(level, frm);
+	printf("map 0x");
+	for (i = 0; i < 10; i++)
+		printf("%02x", cp->map[i]);
+	printf("\n");
+}
+
 static inline void write_link_supervision_timeout_dump(int level, struct frame *frm)
 {
 	write_link_supervision_timeout_cp *cp = frm->ptr;
@@ -1124,7 +1206,6 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_READ_REMOTE_VERSION:
 		case OCF_READ_CLOCK_OFFSET:
 		case OCF_READ_LMP_HANDLE:
-		case OCF_SETUP_SYNC_CONN:
 			generic_command_dump(level + 1, frm);
 			return;
 		case OCF_MASTER_LINK_KEY:
@@ -1136,6 +1217,9 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_REMOTE_NAME_REQ:
 			remote_name_req_dump(level + 1, frm);
 			return;
+		case OCF_SETUP_SYNC_CONN:
+			setup_sync_conn_dump(level + 1, frm);
+			return;
 		}
 		break;
 
@@ -1157,6 +1241,9 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_SWITCH_ROLE:
 			accept_conn_req_dump(level + 1, frm);
 			return;
+		case OCF_QOS_SETUP:
+			qos_setup_dump(level + 1, frm);
+			return;
 		case OCF_WRITE_LINK_POLICY:
 			write_link_policy_dump(level + 1, frm);
 			return;
@@ -1220,6 +1307,9 @@ static inline void command_dump(int level, struct frame *frm)
 		case OCF_WRITE_AFH_MODE:
 			write_inquiry_mode_dump(level + 1, frm);
 			return;
+		case OCF_SET_AFH_CLASSIFICATION:
+			set_afh_classification_dump(level + 1, frm);
+			return;
 		case OCF_READ_TRANSMIT_POWER_LEVEL:
 			request_transmit_power_level_dump(level + 1, frm);
 			return;
@@ -2479,6 +2569,7 @@ static inline void event_dump(int level, struct frame *frm)
 
 		switch (manufacturer) {
 		case 0:
+		case 37:
 		case 48:
 			frm->ptr += HCI_EVENT_HDR_SIZE;
 			frm->len -= HCI_EVENT_HDR_SIZE;
